package com.ruoyi.system.utils.dbsync.compare;

import com.alibaba.druid.util.StringUtils;
import com.ruoyi.system.utils.dbsync.meta.Column;
import com.ruoyi.system.utils.dbsync.meta.Index;
import com.ruoyi.system.utils.dbsync.meta.MetaData;
import com.ruoyi.system.utils.dbsync.meta.Table;
import com.ruoyi.system.utils.dbsync.util.SqlUtil;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class CompareUnits {
   private MetaData source;
   private MetaData target;
   private List<String> changeSql;

   public CompareUnits(MetaData source, MetaData target) {
      this.source = source;
      this.target = target;
      this.changeSql = new ArrayList();
   }

   public void compare() {
      this.compareTables();
      this.compareKeys();
   }

   private void compareSchema() {
   }

   private void compareTables() {
      Iterator var1 = this.source.getTables().values().iterator();

      while(var1.hasNext()) {
         Table table = (Table)var1.next();
         if (this.target.getTables().get(table.getTableName()) == null) {
            this.changeSql.add(table.getCreateTable() + ";");
         } else {
            this.compareSingelTable(table, (Table)this.target.getTables().get(table.getTableName()));
         }
      }

   }

   private void compareSingelTable(Table sourceTable, Table targetTable) {
      this.compareColumns(sourceTable, targetTable);
   }

   private void compareColumns(Table sourceTable, Table targetTable) {
      String after = null;

      Iterator var4;
      Column column;
      String sql;
      for(var4 = sourceTable.getColumns().values().iterator(); var4.hasNext(); after = column.getName()) {
         column = (Column)var4.next();
         if (targetTable.getColumns().get(column.getName()) == null) {
            sql = "alter table `" + this.target.getSchema() + "`." + targetTable.getTableName() + " add " + column.getName() + " ";
            sql = sql + column.getType() + " ";
            if (column.getIsNull().equals("NO")) {
               sql = sql + "NOT NULL ";
            } else {
               sql = sql + "NULL ";
            }

            if (column.getDefaultValue() != null) {
               sql = sql + "DEFAULT " + (column.getDefaultValue2() == null ? "NULL" : SqlUtil.getDbString(column.getDefaultValue())) + " ";
            }

            if (column.getComment() != null) {
               sql = sql + "COMMENT " + SqlUtil.getDbString(column.getComment()) + " ";
            }

            if (after != null) {
               sql = sql + "after " + after;
            }

            this.changeSql.add(sql + ";");
         } else {
            sql = "alter table `" + this.target.getSchema() + "`." + targetTable.getTableName() + " change " + column.getName() + " ";
            Column sourceColumn = column;
            Column targetColumn = (Column)targetTable.getColumns().get(sourceColumn.getName());
            String sqlExtend = this.compareSingleColumn(sourceColumn, targetColumn);
            if (sqlExtend != null) {
               this.changeSql.add(sql + sqlExtend + ";");
            }
         }
      }

      var4 = targetTable.getColumns().values().iterator();

      while(var4.hasNext()) {
         column = (Column)var4.next();
         if (sourceTable.getColumns().get(column.getName()) == null) {
            sql = "alter table `" + this.target.getSchema() + "`." + targetTable.getTableName() + " drop " + column.getName() + " ";
            this.changeSql.add(sql + ";");
         }
      }

   }

   private String compareSingleColumn(Column sourceColumn, Column targetColumn) {
      new ArrayList();
      if (sourceColumn.equals(targetColumn)) {
         return null;
      } else {
         String changeSql = "";
         if (!sourceColumn.getName().equals(targetColumn.getName())) {
            throw new RuntimeException("the bug in this tool");
         } else {
            changeSql = changeSql + sourceColumn.getName() + " ";
            changeSql = changeSql + sourceColumn.getType() + " ";
            if (sourceColumn.getIsNull().equals("NO")) {
               changeSql = changeSql + "NOT NULL ";
            } else {
               changeSql = changeSql + "NULL ";
            }

            if (sourceColumn.getExtra().toUpperCase().indexOf("AUTO_INCREMENT") != -1) {
               changeSql = changeSql + "AUTO_INCREMENT ";
            }

            if (!StringUtils.isEmpty(sourceColumn.getDefaultValue())) {
               changeSql = changeSql + "DEFAULT " + sourceColumn.getDefaultValue() + " ";
            }

            if (sourceColumn.getComment() != null) {
               changeSql = changeSql + "COMMENT " + SqlUtil.getDbString(sourceColumn.getComment()) + " ";
            }

            return changeSql;
         }
      }
   }

   private void compareKeys() {
      Iterator var1 = this.source.getTables().values().iterator();

      while(var1.hasNext()) {
         Table table = (Table)var1.next();
         if (this.target.getTables().get(table.getTableName()) != null) {
            this.compareSingleKeys(table, (Table)this.target.getTables().get(table.getTableName()));
         }
      }

   }

   private void compareSingleKeys(Table sourceTable, Table targetTable) {
      Iterator var3 = sourceTable.getIndexes().values().iterator();

      while(true) {
         Index index;
         String sql;
         do {
            if (!var3.hasNext()) {
               var3 = targetTable.getIndexes().values().iterator();

               while(var3.hasNext()) {
                  index = (Index)var3.next();
                  if (sourceTable.getIndexes().get(index.getIndexName()) == null) {
                     sql = "alter table `" + this.target.getSchema() + "`." + targetTable.getTableName() + " ";
                     if (index.getIndexName().equals("PRIMARY")) {
                        sql = sql + "drop primary key ";
                     } else {
                        sql = sql + "drop index " + index.getIndexName();
                     }

                     this.changeSql.add(sql + ";");
                  }
               }

               return;
            }

            index = (Index)var3.next();
            sql = "alter table `" + this.target.getSchema() + "`." + targetTable.getTableName() + " ";
         } while(targetTable.getIndexes().get(index.getIndexName()) != null);

         if (index.getIndexName().equals("PRIMARY")) {
            sql = sql + "add primary key ";
         } else if (index.getNotUnique().equals("0")) {
            sql = sql + "add unique " + index.getIndexName() + " ";
         } else {
            sql = sql + "add index " + index.getIndexName() + " ";
         }

         sql = sql + "(`";

         String key;
         for(Iterator var6 = index.getColumns().iterator(); var6.hasNext(); sql = sql + key.trim() + "`,`") {
            key = (String)var6.next();
         }

         sql = sql.substring(0, sql.length() - 2) + ")";
         this.changeSql.add(sql + ";");
      }
   }

   public MetaData getSource() {
      return this.source;
   }

   public void setSource(MetaData source) {
      this.source = source;
   }

   public MetaData getTarget() {
      return this.target;
   }

   public void setTarget(MetaData target) {
      this.target = target;
   }

   public List<String> getChangeSql() {
      return this.changeSql;
   }

   public void setChangeSql(List<String> changeSql) {
      this.changeSql = changeSql;
   }
}
